using System;
using System.Windows.Forms;
using SharpMap.Layers;
using SharpMap;
using Point=SharpMap.Geometries.Point;
using System.Drawing;
using SharpMap.Forms;

namespace WinFormWMSTest
{
	public class MainClass
	{
		private Form form1;
		public static void Main(string[] args)
		{
			 new MainClass();
		}
		
		public MainClass()
		{
			form1 = new Form();
			this.SetupForm();
			form1.SetDesktopBounds(0,0,600,600);
			form1.Visible = true;
			
			Application.Run();
		}
		
		public void SetupForm()
		{
			PictureBox p = new PictureBox();
			p.BackColor = Color.White;
			p.SetBounds(100,100,400,400);
			
			Map m = InitializeMap();
			
			MapImage mi = new MapImage();
			mi.Map = m;
			mi.Map.ZoomToExtents();
			form1.Controls.Add(mi);
		}
		
		public static Map InitializeMap()
        {
            string wmsUrl = "http://www2.demis.nl/mapserver/request.asp";

            Map map = new Map();

            WmsLayer layWms = new WmsLayer("Demis Map", wmsUrl);
            layWms.SpatialReferenceSystem = "EPSG:4326";

            layWms.AddLayer("Bathymetry");
            layWms.AddLayer("Topography");
            layWms.AddLayer("Hillshading");

            layWms.SetImageFormat(layWms.OutputFormats[0]);
            layWms.ContinueOnError = true;
                //Skip rendering the WMS Map if the server couldn't be requested (if set to false such an event would crash the app)
            layWms.TimeOut = 5000; //Set timeout to 5 seconds
            layWms.SRID = 4326;
			
			Console.WriteLine(layWms.GetRequestUrl(layWms.Envelope, map.Size));
            map.Layers.Add(layWms);

            //limit the zoom to 360 degrees width
            map.MaximumZoom = 360;
            map.BackColor = Color.LightGray;

            map.Zoom = 360;
            map.Center = new Point(0, 0);

            return map;
        }
	}
}